@@ -11,9 +11,8 @@
 import traceback
 
 from idds.common import exceptions
-from idds.common.constants import ProcessingStatus, ProcessingLocking
+from idds.common.constants import ProcessingStatus, ProcessingLocking, ReturnCode
 from idds.common.utils import setup_logging, truncate_string
-from idds.core import catalog as core_catalog
 from idds.core import processings as core_processings
 from idds.agents.common.eventbus.event import (EventType,
                                                UpdateTransformEvent,
@@ -21,7 +20,8 @@ from idds.agents.common.eventbus.event import (EventType,
                                                TerminatedProcessingEvent,
                                                SyncProcessingEvent)
 
-from .utils import handle_trigger_processing, is_process_terminated
+from .utils import (handle_trigger_processing,
+                    is_process_terminated)
 from .poller import Poller
 
 setup_logging(__name__)
@@ -32,14 +32,22 @@ class Trigger(Poller):
     Trigger works to trigger to release jobs
     """
 
-    def __init__(self, num_threads=1, poll_period=10, retries=3, retrieve_bulk_size=2,
-                 name='Trigger', message_bulk_size=1000, **kwargs):
-        self.set_max_workers()
-        if hasattr(self, 'trigger_max_number_workers'):
-            self.max_number_workers = int(self.trigger_max_number_workers)
+    def __init__(self, num_threads=1, trigger_max_number_workers=3, max_number_workers=3, poll_period=10, retries=3, retrieve_bulk_size=2,
+                 name='Trigger', message_bulk_size=1000, max_updates_per_round=2000, **kwargs):
+        if trigger_max_number_workers > num_threads:
+            self.max_number_workers = trigger_max_number_workers
+        else:
+            self.max_number_workers = max_number_workers
 
-        num_threads = self.max_number_workers * 2
-        super(Trigger, self).__init__(num_threads=num_threads, name=name, **kwargs)
+        self.set_max_workers()
+
+        num_threads = int(self.max_number_workers)
+        super(Trigger, self).__init__(num_threads=num_threads, name=name, max_number_workers=self.max_number_workers,
+                                      retrieve_bulk_size=retrieve_bulk_size, **kwargs)
+        self.logger.info("num_threads: %s" % num_threads)
+
+        self.max_updates_per_round = max_updates_per_round
+        self.logger.info("max_updates_per_round: %s" % self.max_updates_per_round)
 
         if hasattr(self, 'trigger_max_number_workers'):
             self.max_number_workers = int(self.trigger_max_number_workers)
@@ -69,10 +77,12 @@ class Trigger(Poller):
             if processings:
                 self.logger.info("Main thread get [ToTrigger, Triggering] processings to process: %s" % (str(processings)))
 
+            events = []
             for pr_id in processings:
                 self.logger.info("UpdateProcessingEvent(processing_id: %s)" % pr_id)
                 event = TriggerProcessingEvent(publisher_id=self.id, processing_id=pr_id)
-                self.event_bus.send(event)
+                events.append(event)
+            self.event_bus.send_bulk(events)
 
             return processings
         except exceptions.DatabaseException as ex:
@@ -90,6 +100,7 @@ class Trigger(Poller):
             ret_trigger_processing = handle_trigger_processing(processing,
                                                                self.agent_attributes,
                                                                trigger_new_updates=trigger_new_updates,
+                                                               max_updates_per_round=self.max_updates_per_round,
                                                                logger=self.logger,
                                                                log_prefix=log_prefix)
             process_status, update_contents, ret_msgs, parameters, update_dep_contents_status_name, update_dep_contents_status, new_update_contents, ret_update_transforms = ret_trigger_processing
@@ -166,6 +177,7 @@ class Trigger(Poller):
         return ret
 
     def process_trigger_processing_real(self, event):
+        pro_ret = ReturnCode.Ok.value
         try:
             if event:
                 original_event = event
@@ -174,25 +186,27 @@ class Trigger(Poller):
                 pr = self.get_processing(processing_id=event._processing_id, status=None, locking=True)
                 if not pr:
                     self.logger.error("Cannot find processing for event: %s" % str(event))
+                    pro_ret = ReturnCode.Locked.value
                 else:
                     log_pre = self.get_log_prefix(pr)
                     self.logger.info(log_pre + "process_trigger_processing")
                     ret = self.handle_trigger_processing(pr)
                     # self.logger.info(log_pre + "process_trigger_processing result: %s" % str(ret))
 
-                    new_update_contents = ret.get('new_update_contents', None)
+                    # new_update_contents = ret.get('new_update_contents', None)
                     ret['new_update_contents'] = None
-                    ret_update_contents = ret.get('update_contents', None)
+                    # ret_update_contents = ret.get('update_contents', None)
                     self.update_processing(ret, pr)
 
-                    if new_update_contents or ret_update_contents:
+                    update_transforms = ret.get('update_transforms', None)
+                    if update_transforms:
                         # self.logger.info(log_pre + "update_contents_to_others_by_dep_id")
                         # core_catalog.update_contents_to_others_by_dep_id(request_id=pr['request_id'], transform_id=pr['transform_id'])
                         # self.logger.info(log_pre + "update_contents_to_others_by_dep_id done")
 
                         # core_catalog.delete_contents_update(request_id=pr['request_id'], transform_id=pr['transform_id'])
-                        update_transforms = core_catalog.get_updated_transforms_by_content_status(request_id=pr['request_id'],
-                                                                                                  transform_id=pr['transform_id'])
+                        # update_transforms = get_updated_transforms_by_content_status(request_id=pr['request_id'],
+                        #                                                              transform_id=pr['transform_id'])
                         self.logger.info(log_pre + "update_transforms: %s" % str(update_transforms))
                         for update_transform in update_transforms:
                             if 'transform_id' in update_transform:
@@ -211,6 +225,7 @@ class Trigger(Poller):
                                                           processing_id=pr['processing_id'],
                                                           content=event._content,
                                                           counter=original_event._counter)
+                        event.set_terminating()
                         self.event_bus.send(event)
                     else:
                         if ((event._content and 'has_updates' in event._content and event._content['has_updates'])
@@ -219,21 +234,26 @@ class Trigger(Poller):
                             or ('messages' in ret and ret['messages'])):                # noqa E129
                             self.logger.info(log_pre + "SyncProcessingEvent(processing_id: %s)" % pr['processing_id'])
                             event = SyncProcessingEvent(publisher_id=self.id, processing_id=pr['processing_id'],
+                                                        content=event._content,
                                                         counter=original_event._counter)
                             self.event_bus.send(event)
         except Exception as ex:
             self.logger.error(ex)
             self.logger.error(traceback.format_exc())
+            pro_ret = ReturnCode.Failed.value
+        return pro_ret
 
     def process_trigger_processing(self, event):
         self.number_workers += 1
-        self.process_trigger_processing_real(event)
+        ret = self.process_trigger_processing_real(event)
         self.number_workers -= 1
+        return ret
 
     def process_msg_trigger_processing(self, event):
         self.number_msg_workers += 1
-        self.process_trigger_processing_real(event)
+        ret = self.process_trigger_processing_real(event)
         self.number_msg_workers -= 1
+        return ret
 
     def init_event_function_map(self):
         self.event_func_map = {
@@ -253,6 +273,7 @@ class Trigger(Poller):
         """
         try:
             self.logger.info("Starting main thread")
+            self.init_thread_info()
 
             self.load_plugins()
             self.init()
@@ -261,7 +282,7 @@ class Trigger(Poller):
 
             self.init_event_function_map()
 
-            task = self.create_task(task_func=self.get_trigger_processings, task_output_queue=None, task_args=tuple(), task_kwargs={}, delay_time=60, priority=1)
+            task = self.create_task(task_func=self.get_trigger_processings, task_output_queue=None, task_args=tuple(), task_kwargs={}, delay_time=10, priority=1)
             self.add_task(task)
 
             self.execute()
