@@ -381,7 +381,7 @@ def clean_next_poll_at(status, session=None):
 
 
 @read_session
-def get_transform_input_output_maps(transform_id, input_coll_ids, output_coll_ids, log_coll_ids=[], session=None):
+def get_transform_input_output_maps(transform_id, input_coll_ids, output_coll_ids, log_coll_ids=[], with_sub_map_id=False, session=None):
     """
     Get transform input output maps.
 
@@ -391,9 +391,15 @@ def get_transform_input_output_maps(transform_id, input_coll_ids, output_coll_id
     ret = {}
     for content in contents:
         map_id = content['map_id']
-        if map_id not in ret:
-            ret[map_id] = {'inputs_dependency': [], 'inputs': [], 'outputs': [], 'logs': [], 'others': []}
-
+        if not with_sub_map_id:
+            if map_id not in ret:
+                ret[map_id] = {'inputs_dependency': [], 'inputs': [], 'outputs': [], 'logs': [], 'others': []}
+        else:
+            sub_map_id = content['sub_map_id']
+            if map_id not in ret:
+                ret[map_id] = {}
+            if sub_map_id not in ret[map_id]:
+                ret[map_id][sub_map_id] = {'inputs_dependency': [], 'inputs': [], 'outputs': [], 'logs': [], 'others': []}
         """
         if content['coll_id'] in input_coll_ids:
             ret[map_id]['inputs'].append(content)
@@ -404,16 +410,28 @@ def get_transform_input_output_maps(transform_id, input_coll_ids, output_coll_id
         else:
             ret[map_id]['others'].append(content)
         """
-        if content['content_relation_type'] == ContentRelationType.Input:
-            ret[map_id]['inputs'].append(content)
-        elif content['content_relation_type'] == ContentRelationType.InputDependency:
-            ret[map_id]['inputs_dependency'].append(content)
-        elif content['content_relation_type'] == ContentRelationType.Output:
-            ret[map_id]['outputs'].append(content)
-        elif content['content_relation_type'] == ContentRelationType.Log:
-            ret[map_id]['logs'].append(content)
+        if not with_sub_map_id:
+            if content['content_relation_type'] == ContentRelationType.Input:
+                ret[map_id]['inputs'].append(content)
+            elif content['content_relation_type'] == ContentRelationType.InputDependency:
+                ret[map_id]['inputs_dependency'].append(content)
+            elif content['content_relation_type'] == ContentRelationType.Output:
+                ret[map_id]['outputs'].append(content)
+            elif content['content_relation_type'] == ContentRelationType.Log:
+                ret[map_id]['logs'].append(content)
+            else:
+                ret[map_id]['others'].append(content)
         else:
-            ret[map_id]['others'].append(content)
+            if content['content_relation_type'] == ContentRelationType.Input:
+                ret[map_id][sub_map_id]['inputs'].append(content)
+            elif content['content_relation_type'] == ContentRelationType.InputDependency:
+                ret[map_id][sub_map_id]['inputs_dependency'].append(content)
+            elif content['content_relation_type'] == ContentRelationType.Output:
+                ret[map_id][sub_map_id]['outputs'].append(content)
+            elif content['content_relation_type'] == ContentRelationType.Log:
+                ret[map_id][sub_map_id]['logs'].append(content)
+            else:
+                ret[map_id][sub_map_id]['others'].append(content)
     return ret
 
 
@@ -627,3 +645,13 @@ def get_work_name_to_coll_map(request_id):
                                                                             'workload_id': coll['workload_id'],
                                                                             'scope': coll['scope'], 'name': coll['name']})
     return work_name_to_coll_map
+
+
+@read_session
+def get_num_active_transforms(active_status=None, session=None):
+    return orm_transforms.get_num_active_transforms(active_status=active_status, session=session)
+
+
+@read_session
+def get_active_transforms(active_status=None, session=None):
+    return orm_transforms.get_active_transforms(active_status=active_status, session=session)
