@@ -17,7 +17,7 @@ import datetime
 import re
 import copy
 
-from sqlalchemy import or_
+from sqlalchemy import or_, asc
 from sqlalchemy.exc import DatabaseError, IntegrityError
 
 from idds.common import exceptions
@@ -117,8 +117,8 @@ def update_messages(messages, bulk_size=1000, session=None):
 @read_session
 def retrieve_messages(bulk_size=1000, msg_type=None, status=None, source=None,
                       destination=None, request_id=None, workload_id=None,
-                      transform_id=None, processing_id=None,
-                      retries=None, delay=None, session=None):
+                      transform_id=None, processing_id=None, fetching_id=None,
+                      use_poll_period=False, retries=None, delay=None, session=None):
     """
     Retrieve up to $bulk messages.
 
@@ -144,14 +144,6 @@ def retrieve_messages(bulk_size=1000, msg_type=None, status=None, source=None,
                 msg_type = [msg_type[0], msg_type[0]]
 
         query = session.query(models.Message)
-        if request_id is not None:
-            query = query.with_hint(models.Message, "INDEX(MESSAGES MESSAGES_TYPE_ST_IDX)", 'oracle')
-        elif transform_id:
-            query = query.with_hint(models.Message, "INDEX(MESSAGES MESSAGES_TYPE_ST_TF_IDX)", 'oracle')
-        elif processing_id is not None:
-            query = query.with_hint(models.Message, "INDEX(MESSAGES MESSAGES_TYPE_ST_PR_IDX)", 'oracle')
-        else:
-            query = query.with_hint(models.Message, "INDEX(MESSAGES MESSAGES_TYPE_ST_IDX)", 'oracle')
 
         if msg_type is not None:
             query = query.filter(models.Message.msg_type.in_(msg_type))
@@ -173,6 +165,10 @@ def retrieve_messages(bulk_size=1000, msg_type=None, status=None, source=None,
             query = query.filter_by(retries=retries)
         if delay:
             query = query.filter(models.Message.updated_at < datetime.datetime.utcnow() - datetime.timedelta(seconds=delay))
+        elif use_poll_period:
+            query = query.filter(models.Message.updated_at + models.Message.poll_period <= datetime.datetime.utcnow())
+
+        query = query.order_by(asc(models.Message.updated_at))
 
         if bulk_size:
             query = query.order_by(models.Message.created_at).limit(bulk_size)
@@ -181,7 +177,8 @@ def retrieve_messages(bulk_size=1000, msg_type=None, status=None, source=None,
         tmp = query.all()
         if tmp:
             for t in tmp:
-                messages.append(t.to_dict())
+                message = t.to_dict()
+                messages.append(message)
         return messages
     except IntegrityError as e:
         raise exceptions.DatabaseException(e.args)
@@ -201,13 +198,23 @@ def delete_messages(messages, session=None):
     try:
         if message_condition:
             session.query(models.Message).\
-                with_hint(models.Message, "index(messages MESSAGES_PK)", 'oracle').\
                 filter(or_(*message_condition)).\
                 delete(synchronize_session=False)
     except IntegrityError as e:
         raise exceptions.DatabaseException(e.args)
 
 
+@transactional_session
+def clean_old_messages(request_id, session=None):
+    """
+    Delete messages whose request id is older than request_id.
+
+    :param request_id: request id..
+    """
+    session.query(models.Message)\
+           .filter(models.Message.request_id <= request_id)\
+           .delete(synchronize_session=False)
+
 # @transactional_session
 # def update_messages(messages, session=None):
 #     """
