@@ -16,7 +16,7 @@ operations related to Transform.
 import datetime
 
 import sqlalchemy
-from sqlalchemy import and_
+from sqlalchemy import and_, func
 from sqlalchemy.exc import DatabaseError, IntegrityError
 from sqlalchemy.sql.expression import asc, desc
 
@@ -199,8 +199,8 @@ def get_transform_by_id_status(transform_id, status=None, locking=False, session
     """
 
     try:
-        query = session.query(models.Transform).with_hint(models.Transform, "INDEX(TRANSFORMS TRANSFORMS_PK)", 'oracle')\
-                                               .filter(models.Transform.transform_id == transform_id)
+        query = session.query(models.Transform)\
+                       .filter(models.Transform.transform_id == transform_id)
 
         if status:
             if not isinstance(status, (list, tuple)):
@@ -509,3 +509,41 @@ def clean_next_poll_at(status, session=None):
     params = {'next_poll_at': datetime.datetime.utcnow()}
     session.query(models.Transform).filter(models.Transform.status.in_(status))\
            .update(params, synchronize_session=False)
+
+
+@read_session
+def get_num_active_transforms(active_status=None, session=None):
+    if active_status and not isinstance(active_status, (list, tuple)):
+        active_status = [active_status]
+    if active_status and len(active_status) == 1:
+        active_status = [active_status[0], active_status[0]]
+
+    try:
+        query = session.query(models.Transform.status, models.Transform.site, func.count(models.Transform.transform_id))
+        if active_status:
+            query = query.filter(models.Transform.status.in_(active_status))
+        query = query.group_by(models.Transform.status, models.Transform.site)
+        tmp = query.all()
+        return tmp
+    except Exception as error:
+        raise error
+
+
+@read_session
+def get_active_transforms(active_status=None, session=None):
+    if active_status and not isinstance(active_status, (list, tuple)):
+        active_status = [active_status]
+    if active_status and len(active_status) == 1:
+        active_status = [active_status[0], active_status[0]]
+
+    try:
+        query = session.query(models.Transform.request_id,
+                              models.Transform.transform_id,
+                              models.Transform.site,
+                              models.Transform.status)
+        if active_status:
+            query = query.filter(models.Transform.status.in_(active_status))
+        tmp = query.all()
+        return tmp
+    except Exception as error:
+        raise error
