@@ -6,17 +6,20 @@
 # http://www.apache.org/licenses/LICENSE-2.0OA
 #
 # Authors:
-# - Wen Guan, <wen.guan@cern.ch>, 2019
+# - Wen Guan, <wen.guan@cern.ch>, 2019 - 2023
 
 """----------------------
    Web service app
 ----------------------"""
 
+import logging
+import sys
 import flask
 from flask import Flask, Response
 
 from idds.common import exceptions
 from idds.common.authentication import authenticate_x509, authenticate_oidc, authenticate_is_super_user
+from idds.common.config import (config_has_section, config_has_option, config_get)
 from idds.common.constants import HTTP_STATUS_CODE
 from idds.common.utils import get_rest_debug
 # from idds.common.utils import get_rest_url_prefix
@@ -147,7 +150,21 @@ def after_request(response):
     return response
 
 
+def setup_logging(loglevel=None):
+    if loglevel is None:
+        if config_has_section('common') and config_has_option('common', 'loglevel'):
+            loglevel = getattr(logging, config_get('common', 'loglevel').upper())
+        else:
+            loglevel = logging.INFO
+
+    logging.basicConfig(stream=sys.stdout, level=loglevel,
+                        format='%(asctime)s\t%(threadName)s\t%(name)s\t%(levelname)s\t%(message)s')
+
+
 def create_app(auth_type=None):
+
+    setup_logging()
+
     # url_prefix = get_rest_url_prefix()
     application = Flask(__name__)
 
