@@ -132,8 +132,9 @@ def print_workflow(workflow, layers=0):
                 print(prefix + "   or: " + str(work.or_custom_conditions) + " and: " + str(work.and_custom_conditions))
                 print(prefix + "   output: " + str(work.output_data))
                 print(prefix + "   " + workflow.runs[run].works[work_id].task_name + ", num_run: " + str(workflow.runs[run].works[work_id].num_run))
-                # print(prefix + "   is_terminated: " + str(workflow.runs[run].works[work_id].is_terminated()))
-                # print(prefix + "   is_finished: " + str(workflow.runs[run].works[work_id].is_finished()))
+                print(prefix + "   workload_id: " + str(work.workload_id))
+                print(prefix + "   is_terminated: " + str(workflow.runs[run].works[work_id].is_terminated()))
+                print(prefix + "   is_finished: " + str(workflow.runs[run].works[work_id].is_finished()))
         if workflow.runs[run].has_loop_condition():
             print(prefix + " Loop condition status: %s" % workflow.runs[run].get_loop_condition_status())
             print(prefix + " Loop condition: %s" % json_dumps(workflow.runs[run].loop_condition, sort_keys=True, indent=4))
@@ -171,7 +172,10 @@ def print_workflow_template(workflow, layers=0):
 # reqs = get_requests(request_id=381520, with_request=True, with_detail=False, with_metadata=True)
 # reqs = get_requests(request_id=28182323, with_request=True, with_detail=False, with_metadata=True)
 # reqs = get_requests(request_id=385554, with_request=True, with_detail=False, with_metadata=True)
-reqs = get_requests(request_id=474, with_request=True, with_detail=False, with_metadata=True)
+reqs = get_requests(request_id=479187, with_request=True, with_detail=False, with_metadata=True)
+reqs = get_requests(request_id=4498, with_request=True, with_detail=False, with_metadata=True)
+reqs = get_requests(request_id=4615, with_request=True, with_detail=False, with_metadata=True)
+reqs = get_requests(request_id=535793, with_request=True, with_detail=False, with_metadata=True)
 for req in reqs:
     # print(req['request_id'])
     # print(req)
@@ -179,11 +183,18 @@ for req in reqs:
     # print(json_dumps(req, sort_keys=True, indent=4))
     # show_works(req)
     pass
-    workflow = req['request_metadata']['build_workflow']
-    # workflow.get_new_works()
-    print(workflow.runs.keys())
-    # print(workflow.runs["1"])
-    print(json_dumps(workflow.runs["1"], sort_keys=True, indent=4))
+    if 'build_workflow' in req['request_metadata']:
+        workflow = req['request_metadata']['build_workflow']
+        # workflow.get_new_works()
+        print(workflow.runs.keys())
+        # print(workflow.runs["1"])
+        print(json_dumps(workflow.runs["1"], sort_keys=True, indent=4))
+    elif 'workflow' in req['request_metadata']:
+        workflow = req['request_metadata']['workflow']
+        # workflow.get_new_works()
+        print(workflow.runs.keys())
+        # print(workflow.runs["1"])
+        # print(json_dumps(workflow.runs["1"], sort_keys=True, indent=4))
 
     # print(workflow.runs["1"].works.keys())
     # print(workflow.runs["1"].has_loop_condition())
@@ -208,10 +219,15 @@ for req in reqs:
         print("work %s signature: %s" % (work.get_work_id(), work.signature))
 
     # print("workflow template")
-    # print_workflow_template(workflow)
+    print_workflow_template(workflow)
 
     # workflow.sync_works()
 
+    print("workflow template")
+    print(json_dumps(workflow.template, sort_keys=True, indent=4))
+
+
+"""
 sys.exit(0)
 
 reqs = get_requests(request_id=28182323, with_request=False, with_detail=True, with_metadata=False)
@@ -219,6 +235,7 @@ for req in reqs:
     print(json_dumps(req, sort_keys=True, indent=4))
 
 # sys.exit(0)
+"""
 
 """
 # reqs = get_requests()
@@ -234,7 +251,7 @@ sys.exit(0)
 
 """
 
-
+"""
 tfs = get_transforms(request_id=470)
 # tfs = get_transforms(transform_id=350723)
 for tf in tfs:
@@ -250,7 +267,7 @@ for tf in tfs:
     pass
 
 sys.exit(0)
-
+"""
 
 """
 msgs = retrieve_messages(workload_id=25972557)
@@ -271,7 +288,7 @@ print(number_contents)
 sys.exit(0)
 """
 
-prs = get_processings(request_id=373602)
+prs = get_processings(request_id=4615)
 # prs = get_processings(transform_id=350723)
 i = 0
 for pr in prs:
