@@ -15,8 +15,9 @@ Test client.
 
 
 from idds.client.clientmanager import ClientManager
+from idds.common.constants import RequestStatus
 from idds.common.utils import json_dumps  # noqa F401
-from idds.rest.v1.utils import convert_old_req_2_workflow_req
+from idds.rest.v1.utils import convert_old_req_2_workflow_req  # noqa F401
 from idds.common.utils import setup_logging
 
 
@@ -37,10 +38,13 @@ def migrate():
 
     slac_k8s_dev_host = 'https://rubin-panda-idds-dev.slac.stanford.edu:8443/idds'  # noqa F841
 
+    slac_k8s_prod_host = 'https://usdf-panda-idds.slac.stanford.edu:8443/idds'  # noqa F841
+
     cern_k8s_dev_host = 'https://panda-idds-dev.cern.ch/idds'  # noqa F841
 
     # cm1 = ClientManager(host=atlas_host)
-    cm1 = ClientManager(host=doma_host)
+    # cm1 = ClientManager(host=doma_host)
+    cm1 = ClientManager(host=atlas_host)
     # cm1 = ClientManager(host=slac_k8s_dev_host)
     # reqs = cm1.get_requests(request_id=290)
     # old_request_id = 298163
@@ -54,18 +58,32 @@ def migrate():
     old_request_id = 2802
     old_request_id = 2816       # big tasks
     # old_request_id = 3178     # 125 tasks
+    old_request_id = 3578
+    old_request_id = 3612
+    old_request_id = 3628
 
+    old_request_ids = [3628]
+
+    # old_request_ids = [21]
+
+    old_request_ids = [551889]
+
+    old_request_ids = [i for i in range(551911, 556618)]
+    old_request_ids = [i for i in range(556618, 556712)]
+    old_request_ids = [i for i in range(556712, 556940)]
+    old_request_ids = [i for i in range(556940, 557110)]
     # old_request_id = 1
     # for old_request_id in [152]:
     # for old_request_id in [60]:    # noqa E115
     # for old_request_id in [200]:    # noqa E115
-    for old_request_id in [old_request_id]:    # noqa E115  # doma 183
+    for old_request_id in old_request_ids:    # noqa E115  # doma 183
         reqs = cm1.get_requests(request_id=old_request_id, with_metadata=True)
 
-        # cm2 = ClientManager(host=dev_host)
+        cm2 = ClientManager(host=dev_host)
         # cm2 = ClientManager(host=doma_host)
         # cm2 = ClientManager(host=atlas_host)
-        cm2 = ClientManager(host=slac_k8s_dev_host)
+        # cm2 = ClientManager(host=slac_k8s_dev_host)
+        # cm2 = ClientManager(host=slac_k8s_prod_host)
         # cm2 = ClientManager(host=cern_k8s_dev_host)
         # print(reqs)
 
@@ -75,7 +93,12 @@ def migrate():
             # workflow = req['request_metadata']['workflow']
             # print(json_dumps(workflow, sort_keys=True, indent=4))
 
-            req = convert_old_req_2_workflow_req(req)
+            # req = convert_old_req_2_workflow_req(req)
+            print(req['status'])
+            if req['status'] in [RequestStatus.Finished]:
+                print('request stutus: %s, skip' % str(req['status']))
+                continue
+
             workflow = req['request_metadata']['workflow']
             workflow.clean_works()
 
